@@ -68,7 +68,6 @@ extern void groups_free(struct group_info *);
 extern int set_current_groups(struct group_info *);
 extern int set_groups(struct cred *, struct group_info *);
 extern int groups_search(const struct group_info *, kgid_t);
-extern bool may_setgroups(void);
 
 /* access the groups "array" with this macro */
 #define GROUP_AT(gi, i) \
@@ -345,10 +344,7 @@ extern struct user_namespace init_user_ns;
 #ifdef CONFIG_USER_NS
 #define current_user_ns()	(current_cred_xxx(user_ns))
 #else
-static inline struct user_namespace *current_user_ns(void)
-{
-	return &init_user_ns;
-}
+#define current_user_ns()	(&init_user_ns)
 #endif
 
 
