@@ -1,4 +1,15 @@
- 
+/*
+ * wm8350.c -- WM8350 ALSA SoC audio driver
+ *
+ * Copyright (C) 2007-12 Wolfson Microelectronics PLC.
+ *
+ * Author: Liam Girdwood <lrg@slimlogic.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ */
+
 #include <linux/module.h>
 #include <linux/moduleparam.h>
 #include <linux/init.h>
@@ -25,6 +36,9 @@
 #define WM8350_RAMP_UP		1
 #define WM8350_RAMP_DOWN	2
 
+/* We only include the analogue supplies here; the digital supplies
+ * need to be available well before this driver can be probed.
+ */
 static const char *supply_names[] = {
 	"AVDD",
 	"HPVDD",
@@ -57,6 +71,9 @@ struct wm8350_data {
 	int fll_freq_in;
 };
 
+/*
+ * Ramp OUT1 PGA volume to minimise pops at stream startup and shutdown.
+ */
 static inline int wm8350_out1_ramp_step(struct snd_soc_codec *codec)
 {
 	struct wm8350_data *wm8350_data = snd_soc_codec_get_drvdata(codec);
@@ -65,11 +82,12 @@ static inline int wm8350_out1_ramp_step(struct snd_soc_codec *codec)
 	int left_complete = 0, right_complete = 0;
 	u16 reg, val;
 
+	/* left channel */
 	reg = wm8350_reg_read(wm8350, WM8350_LOUT1_VOLUME);
 	val = (reg & WM8350_OUT1L_VOL_MASK) >> WM8350_OUT1L_VOL_SHIFT;
 
 	if (out1->ramp == WM8350_RAMP_UP) {
-		 
+		/* ramp step up */
 		if (val < out1->left_vol) {
 			val++;
 			reg &= ~WM8350_OUT1L_VOL_MASK;
@@ -78,7 +96,7 @@ static inline int wm8350_out1_ramp_step(struct snd_soc_codec *codec)
 		} else
 			left_complete = 1;
 	} else if (out1->ramp == WM8350_RAMP_DOWN) {
-		 
+		/* ramp step down */
 		if (val > 0) {
 			val--;
 			reg &= ~WM8350_OUT1L_VOL_MASK;
@@ -89,10 +107,11 @@ static inline int wm8350_out1_ramp_step(struct snd_soc_codec *codec)
 	} else
 		return 1;
 
+	/* right channel */
 	reg = wm8350_reg_read(wm8350, WM8350_ROUT1_VOLUME);
 	val = (reg & WM8350_OUT1R_VOL_MASK) >> WM8350_OUT1R_VOL_SHIFT;
 	if (out1->ramp == WM8350_RAMP_UP) {
-		 
+		/* ramp step up */
 		if (val < out1->right_vol) {
 			val++;
 			reg &= ~WM8350_OUT1R_VOL_MASK;
@@ -101,7 +120,7 @@ static inline int wm8350_out1_ramp_step(struct snd_soc_codec *codec)
 		} else
 			right_complete = 1;
 	} else if (out1->ramp == WM8350_RAMP_DOWN) {
-		 
+		/* ramp step down */
 		if (val > 0) {
 			val--;
 			reg &= ~WM8350_OUT1R_VOL_MASK;
@@ -111,12 +130,16 @@ static inline int wm8350_out1_ramp_step(struct snd_soc_codec *codec)
 			right_complete = 1;
 	}
 
+	/* only hit the update bit if either volume has changed this step */
 	if (!left_complete || !right_complete)
 		wm8350_set_bits(wm8350, WM8350_LOUT1_VOLUME, WM8350_OUT1_VU);
 
 	return left_complete & right_complete;
 }
 
+/*
+ * Ramp OUT2 PGA volume to minimise pops at stream startup and shutdown.
+ */
 static inline int wm8350_out2_ramp_step(struct snd_soc_codec *codec)
 {
 	struct wm8350_data *wm8350_data = snd_soc_codec_get_drvdata(codec);
@@ -125,10 +148,11 @@ static inline int wm8350_out2_ramp_step(struct snd_soc_codec *codec)
 	int left_complete = 0, right_complete = 0;
 	u16 reg, val;
 
+	/* left channel */
 	reg = wm8350_reg_read(wm8350, WM8350_LOUT2_VOLUME);
 	val = (reg & WM8350_OUT2L_VOL_MASK) >> WM8350_OUT1L_VOL_SHIFT;
 	if (out2->ramp == WM8350_RAMP_UP) {
-		 
+		/* ramp step up */
 		if (val < out2->left_vol) {
 			val++;
 			reg &= ~WM8350_OUT2L_VOL_MASK;
@@ -137,7 +161,7 @@ static inline int wm8350_out2_ramp_step(struct snd_soc_codec *codec)
 		} else
 			left_complete = 1;
 	} else if (out2->ramp == WM8350_RAMP_DOWN) {
-		 
+		/* ramp step down */
 		if (val > 0) {
 			val--;
 			reg &= ~WM8350_OUT2L_VOL_MASK;
@@ -148,10 +172,11 @@ static inline int wm8350_out2_ramp_step(struct snd_soc_codec *codec)
 	} else
 		return 1;
 
+	/* right channel */
 	reg = wm8350_reg_read(wm8350, WM8350_ROUT2_VOLUME);
 	val = (reg & WM8350_OUT2R_VOL_MASK) >> WM8350_OUT1R_VOL_SHIFT;
 	if (out2->ramp == WM8350_RAMP_UP) {
-		 
+		/* ramp step up */
 		if (val < out2->right_vol) {
 			val++;
 			reg &= ~WM8350_OUT2R_VOL_MASK;
@@ -160,7 +185,7 @@ static inline int wm8350_out2_ramp_step(struct snd_soc_codec *codec)
 		} else
 			right_complete = 1;
 	} else if (out2->ramp == WM8350_RAMP_DOWN) {
-		 
+		/* ramp step down */
 		if (val > 0) {
 			val--;
 			reg &= ~WM8350_OUT2R_VOL_MASK;
@@ -170,12 +195,19 @@ static inline int wm8350_out2_ramp_step(struct snd_soc_codec *codec)
 			right_complete = 1;
 	}
 
+	/* only hit the update bit if either volume has changed this step */
 	if (!left_complete || !right_complete)
 		wm8350_set_bits(wm8350, WM8350_LOUT2_VOLUME, WM8350_OUT2_VU);
 
 	return left_complete & right_complete;
 }
 
+/*
+ * This work ramps both output PGAs at stream start/stop time to
+ * minimise pop associated with DAPM power switching.
+ * It's best to enable Zero Cross when ramping occurs to minimise any
+ * zipper noises.
+ */
 static void wm8350_pga_work(struct work_struct *work)
 {
 	struct snd_soc_dapm_context *dapm =
@@ -186,9 +218,11 @@ static void wm8350_pga_work(struct work_struct *work)
 	    *out2 = &wm8350_data->out2;
 	int i, out1_complete, out2_complete;
 
+	/* do we need to ramp at all ? */
 	if (out1->ramp == WM8350_RAMP_NONE && out2->ramp == WM8350_RAMP_NONE)
 		return;
 
+	/* PGA volumes have 6 bits of resolution to ramp */
 	for (i = 0; i <= 63; i++) {
 		out1_complete = 1, out2_complete = 1;
 		if (out1->ramp != WM8350_RAMP_NONE)
@@ -196,12 +230,14 @@ static void wm8350_pga_work(struct work_struct *work)
 		if (out2->ramp != WM8350_RAMP_NONE)
 			out2_complete = wm8350_out2_ramp_step(codec);
 
+		/* ramp finished ? */
 		if (out1_complete && out2_complete)
 			break;
 
+		/* we need to delay longer on the up ramp */
 		if (out1->ramp == WM8350_RAMP_UP ||
 		    out2->ramp == WM8350_RAMP_UP) {
-			 
+			/* delay is longer over 0dB as increases are larger */
 			if (i >= WM8350_OUTn_0dB)
 				schedule_timeout_interruptible(msecs_to_jiffies
 							       (2));
@@ -209,13 +245,17 @@ static void wm8350_pga_work(struct work_struct *work)
 				schedule_timeout_interruptible(msecs_to_jiffies
 							       (1));
 		} else
-			udelay(50);	 
+			udelay(50);	/* doesn't matter if we delay longer */
 	}
 
 	out1->ramp = WM8350_RAMP_NONE;
 	out2->ramp = WM8350_RAMP_NONE;
 }
 
+/*
+ * WM8350 Controls
+ */
+
 static int pga_event(struct snd_soc_dapm_widget *w,
 		     struct snd_kcontrol *kcontrol, int event)
 {
@@ -234,7 +274,7 @@ static int pga_event(struct snd_soc_dapm_widget *w,
 		break;
 
 	default:
-		BUG();
+		WARN(1, "Invalid shift %d\n", w->shift);
 		return -1;
 	}
 
@@ -262,11 +302,7 @@ static int pga_event(struct snd_soc_dapm_widget *w,
 static int wm8350_put_volsw_2r_vu(struct snd_kcontrol *kcontrol,
 				  struct snd_ctl_elem_value *ucontrol)
 {
-#if defined(CONFIG_SYNO_LSP_ARMADA)
-	struct snd_soc_codec *codec = snd_soc_kcontrol_codec(kcontrol);
-#else  
 	struct snd_soc_codec *codec = snd_kcontrol_chip(kcontrol);
-#endif  
 	struct wm8350_data *wm8350_priv = snd_soc_codec_get_drvdata(codec);
 	struct wm8350_output *out = NULL;
 	struct soc_mixer_control *mc =
@@ -275,6 +311,9 @@ static int wm8350_put_volsw_2r_vu(struct snd_kcontrol *kcontrol,
 	unsigned int reg = mc->reg;
 	u16 val;
 
+	/* For OUT1 and OUT2 we shadow the values and only actually write
+	 * them out when active in order to ensure the amplifier comes on
+	 * as quietly as possible. */
 	switch (reg) {
 	case WM8350_LOUT1_VOLUME:
 		out = &wm8350_priv->out1;
@@ -297,6 +336,7 @@ static int wm8350_put_volsw_2r_vu(struct snd_kcontrol *kcontrol,
 	if (ret < 0)
 		return ret;
 
+	/* now hit the volume update bits (always bit 8) */
 	val = snd_soc_read(codec, reg);
 	snd_soc_write(codec, reg, val | WM8350_OUT1_VU);
 	return 1;
@@ -305,11 +345,7 @@ static int wm8350_put_volsw_2r_vu(struct snd_kcontrol *kcontrol,
 static int wm8350_get_volsw_2r(struct snd_kcontrol *kcontrol,
 			       struct snd_ctl_elem_value *ucontrol)
 {
-#if defined(CONFIG_SYNO_LSP_ARMADA)
-	struct snd_soc_codec *codec = snd_soc_kcontrol_codec(kcontrol);
-#else  
 	struct snd_soc_codec *codec = snd_kcontrol_chip(kcontrol);
-#endif  
 	struct wm8350_data *wm8350_priv = snd_soc_codec_get_drvdata(codec);
 	struct wm8350_output *out1 = &wm8350_priv->out1;
 	struct wm8350_output *out2 = &wm8350_priv->out2;
@@ -317,6 +353,7 @@ static int wm8350_get_volsw_2r(struct snd_kcontrol *kcontrol,
 		(struct soc_mixer_control *)kcontrol->private_value;
 	unsigned int reg = mc->reg;
 
+	/* If these are cached registers use the cache */
 	switch (reg) {
 	case WM8350_LOUT1_VOLUME:
 		ucontrol->value.integer.value[0] = out1->left_vol;
@@ -448,6 +485,11 @@ static const struct snd_kcontrol_new wm8350_snd_controls[] = {
 		     14, 1, 1),
 };
 
+/*
+ * DAPM Controls
+ */
+
+/* Left Playback Mixer */
 static const struct snd_kcontrol_new wm8350_left_play_mixer_controls[] = {
 	SOC_DAPM_SINGLE("Playback Switch",
 			WM8350_LEFT_MIXER_CONTROL, 11, 1, 0),
@@ -461,6 +503,7 @@ static const struct snd_kcontrol_new wm8350_left_play_mixer_controls[] = {
 			WM8350_LEFT_MIXER_CONTROL, 1, 1, 0),
 };
 
+/* Right Playback Mixer */
 static const struct snd_kcontrol_new wm8350_right_play_mixer_controls[] = {
 	SOC_DAPM_SINGLE("Playback Switch",
 			WM8350_RIGHT_MIXER_CONTROL, 12, 1, 0),
@@ -474,6 +517,7 @@ static const struct snd_kcontrol_new wm8350_right_play_mixer_controls[] = {
 			WM8350_RIGHT_MIXER_CONTROL, 1, 1, 0),
 };
 
+/* Out4 Mixer */
 static const struct snd_kcontrol_new wm8350_out4_mixer_controls[] = {
 	SOC_DAPM_SINGLE("Right Playback Switch",
 			WM8350_OUT4_MIXER_CONTROL, 12, 1, 0),
@@ -489,6 +533,7 @@ static const struct snd_kcontrol_new wm8350_out4_mixer_controls[] = {
 			WM8350_OUT4_MIXER_CONTROL, 0, 1, 0),
 };
 
+/* Out3 Mixer */
 static const struct snd_kcontrol_new wm8350_out3_mixer_controls[] = {
 	SOC_DAPM_SINGLE("Left Playback Switch",
 			WM8350_OUT3_MIXER_CONTROL, 11, 1, 0),
@@ -500,6 +545,7 @@ static const struct snd_kcontrol_new wm8350_out3_mixer_controls[] = {
 			WM8350_OUT3_MIXER_CONTROL, 0, 1, 0),
 };
 
+/* Left Input Mixer */
 static const struct snd_kcontrol_new wm8350_left_capt_mixer_controls[] = {
 	SOC_DAPM_SINGLE_TLV("L2 Capture Volume",
 			    WM8350_INPUT_MIXER_VOLUME_L, 1, 7, 0, out_mix_tlv),
@@ -509,6 +555,7 @@ static const struct snd_kcontrol_new wm8350_left_capt_mixer_controls[] = {
 			WM8350_LEFT_INPUT_VOLUME, 14, 1, 1),
 };
 
+/* Right Input Mixer */
 static const struct snd_kcontrol_new wm8350_right_capt_mixer_controls[] = {
 	SOC_DAPM_SINGLE_TLV("L2 Capture Volume",
 			    WM8350_INPUT_MIXER_VOLUME_R, 5, 7, 0, out_mix_tlv),
@@ -518,21 +565,25 @@ static const struct snd_kcontrol_new wm8350_right_capt_mixer_controls[] = {
 			WM8350_RIGHT_INPUT_VOLUME, 14, 1, 1),
 };
 
+/* Left Mic Mixer */
 static const struct snd_kcontrol_new wm8350_left_mic_mixer_controls[] = {
 	SOC_DAPM_SINGLE("INN Capture Switch", WM8350_INPUT_CONTROL, 1, 1, 0),
 	SOC_DAPM_SINGLE("INP Capture Switch", WM8350_INPUT_CONTROL, 0, 1, 0),
 	SOC_DAPM_SINGLE("IN2 Capture Switch", WM8350_INPUT_CONTROL, 2, 1, 0),
 };
 
+/* Right Mic Mixer */
 static const struct snd_kcontrol_new wm8350_right_mic_mixer_controls[] = {
 	SOC_DAPM_SINGLE("INN Capture Switch", WM8350_INPUT_CONTROL, 9, 1, 0),
 	SOC_DAPM_SINGLE("INP Capture Switch", WM8350_INPUT_CONTROL, 8, 1, 0),
 	SOC_DAPM_SINGLE("IN2 Capture Switch", WM8350_INPUT_CONTROL, 10, 1, 0),
 };
 
+/* Beep Switch */
 static const struct snd_kcontrol_new wm8350_beep_switch_controls =
 SOC_DAPM_SINGLE("Switch", WM8350_BEEP_VOLUME, 15, 1, 1);
 
+/* Out4 Capture Mux */
 static const struct snd_kcontrol_new wm8350_out4_capture_controls =
 SOC_DAPM_ENUM("Route", wm8350_enum[7]);
 
@@ -585,6 +636,7 @@ static const struct snd_soc_dapm_widget wm8350_dapm_widgets[] = {
 			   &wm8350_right_mic_mixer_controls[0],
 			   ARRAY_SIZE(wm8350_right_mic_mixer_controls)),
 
+	/* virtual mixer for Beep and Out2R */
 	SND_SOC_DAPM_MIXER("Out2 Mixer", SND_SOC_NOPM, 0, 0, NULL, 0),
 
 	SND_SOC_DAPM_SWITCH("Beep", WM8350_POWER_MGMT_3, 7, 0,
@@ -623,18 +675,21 @@ static const struct snd_soc_dapm_widget wm8350_dapm_widgets[] = {
 
 static const struct snd_soc_dapm_route wm8350_dapm_routes[] = {
 
+	/* left playback mixer */
 	{"Left Playback Mixer", "Playback Switch", "Left DAC"},
 	{"Left Playback Mixer", "Left Bypass Switch", "IN3L PGA"},
 	{"Left Playback Mixer", "Right Playback Switch", "Right DAC"},
 	{"Left Playback Mixer", "Left Sidetone Switch", "Left Mic Mixer"},
 	{"Left Playback Mixer", "Right Sidetone Switch", "Right Mic Mixer"},
 
+	/* right playback mixer */
 	{"Right Playback Mixer", "Playback Switch", "Right DAC"},
 	{"Right Playback Mixer", "Right Bypass Switch", "IN3R PGA"},
 	{"Right Playback Mixer", "Left Playback Switch", "Left DAC"},
 	{"Right Playback Mixer", "Left Sidetone Switch", "Left Mic Mixer"},
 	{"Right Playback Mixer", "Right Sidetone Switch", "Right Mic Mixer"},
 
+	/* out4 playback mixer */
 	{"Out4 Mixer", "Right Playback Switch", "Right DAC"},
 	{"Out4 Mixer", "Left Playback Switch", "Left DAC"},
 	{"Out4 Mixer", "Right Capture Switch", "Right Capture Mixer"},
@@ -643,48 +698,59 @@ static const struct snd_soc_dapm_route wm8350_dapm_routes[] = {
 	{"Out4 Mixer", "Left Mixer Switch", "Left Playback Mixer"},
 	{"OUT4", NULL, "Out4 Mixer"},
 
+	/* out3 playback mixer */
 	{"Out3 Mixer", "Left Playback Switch", "Left DAC"},
 	{"Out3 Mixer", "Left Capture Switch", "Left Capture Mixer"},
 	{"Out3 Mixer", "Left Mixer Switch", "Left Playback Mixer"},
 	{"Out3 Mixer", "Out4 Playback Switch", "Out4 Mixer"},
 	{"OUT3", NULL, "Out3 Mixer"},
 
+	/* out2 */
 	{"Right Out2 PGA", NULL, "Right Playback Mixer"},
 	{"Left Out2 PGA", NULL, "Left Playback Mixer"},
 	{"OUT2L", NULL, "Left Out2 PGA"},
 	{"OUT2R", NULL, "Right Out2 PGA"},
 
+	/* out1 */
 	{"Right Out1 PGA", NULL, "Right Playback Mixer"},
 	{"Left Out1 PGA", NULL, "Left Playback Mixer"},
 	{"OUT1L", NULL, "Left Out1 PGA"},
 	{"OUT1R", NULL, "Right Out1 PGA"},
 
+	/* ADCs */
 	{"Left ADC", NULL, "Left Capture Mixer"},
 	{"Right ADC", NULL, "Right Capture Mixer"},
 
+	/* Left capture mixer */
 	{"Left Capture Mixer", "L2 Capture Volume", "IN2L"},
 	{"Left Capture Mixer", "L3 Capture Volume", "IN3L PGA"},
 	{"Left Capture Mixer", "PGA Capture Switch", "Left Mic Mixer"},
 	{"Left Capture Mixer", NULL, "Out4 Capture Channel"},
 
+	/* Right capture mixer */
 	{"Right Capture Mixer", "L2 Capture Volume", "IN2R"},
 	{"Right Capture Mixer", "L3 Capture Volume", "IN3R PGA"},
 	{"Right Capture Mixer", "PGA Capture Switch", "Right Mic Mixer"},
 	{"Right Capture Mixer", NULL, "Out4 Capture Channel"},
 
+	/* L3 Inputs */
 	{"IN3L PGA", NULL, "IN3L"},
 	{"IN3R PGA", NULL, "IN3R"},
 
+	/* Left Mic mixer */
 	{"Left Mic Mixer", "INN Capture Switch", "IN1LN"},
 	{"Left Mic Mixer", "INP Capture Switch", "IN1LP"},
 	{"Left Mic Mixer", "IN2 Capture Switch", "IN2L"},
 
+	/* Right Mic mixer */
 	{"Right Mic Mixer", "INN Capture Switch", "IN1RN"},
 	{"Right Mic Mixer", "INP Capture Switch", "IN1RP"},
 	{"Right Mic Mixer", "IN2 Capture Switch", "IN2R"},
 
+	/* out 4 capture */
 	{"Out4 Capture Channel", NULL, "Out4 Mixer"},
 
+	/* Beep */
 	{"Beep", NULL, "IN3R PGA"},
 };
 
@@ -713,6 +779,7 @@ static int wm8350_set_dai_sysclk(struct snd_soc_dai *codec_dai,
 		break;
 	}
 
+	/* MCLK direction */
 	if (dir == SND_SOC_CLOCK_OUT)
 		wm8350_set_bits(wm8350, WM8350_CLOCK_CONTROL_2,
 				WM8350_MCLK_DIR);
@@ -783,6 +850,7 @@ static int wm8350_set_dai_fmt(struct snd_soc_dai *codec_dai, unsigned int fmt)
 	u16 adc_lrc = snd_soc_read(codec, WM8350_ADC_LR_RATE) &
 	    ~WM8350_ADCLRC_ENA;
 
+	/* set master/slave audio interface */
 	switch (fmt & SND_SOC_DAIFMT_MASTER_MASK) {
 	case SND_SOC_DAIFMT_CBM_CFM:
 		master |= WM8350_BCLK_MSTR;
@@ -795,6 +863,7 @@ static int wm8350_set_dai_fmt(struct snd_soc_dai *codec_dai, unsigned int fmt)
 		return -EINVAL;
 	}
 
+	/* interface format */
 	switch (fmt & SND_SOC_DAIFMT_FORMAT_MASK) {
 	case SND_SOC_DAIFMT_I2S:
 		iface |= 0x2 << 8;
@@ -814,6 +883,7 @@ static int wm8350_set_dai_fmt(struct snd_soc_dai *codec_dai, unsigned int fmt)
 		return -EINVAL;
 	}
 
+	/* clock inversion */
 	switch (fmt & SND_SOC_DAIFMT_INV_MASK) {
 	case SND_SOC_DAIFMT_NB_NF:
 		break;
@@ -847,6 +917,7 @@ static int wm8350_pcm_hw_params(struct snd_pcm_substream *substream,
 	u16 iface = snd_soc_read(codec, WM8350_AI_FORMATING) &
 	    ~WM8350_AIF_WL_MASK;
 
+	/* bit size */
 	switch (params_format(params)) {
 	case SNDRV_PCM_FORMAT_S16_LE:
 		break;
@@ -863,6 +934,9 @@ static int wm8350_pcm_hw_params(struct snd_pcm_substream *substream,
 
 	snd_soc_write(codec, WM8350_AI_FORMATING, iface);
 
+	/* The sloping stopband filter is recommended for use with
+	 * lower sample rates to improve performance.
+	 */
 	if (substream->stream == SNDRV_PCM_STREAM_PLAYBACK) {
 		if (params_rate(params) < 24000)
 			wm8350_set_bits(wm8350, WM8350_DAC_MUTE_VOLUME,
@@ -890,13 +964,16 @@ static int wm8350_mute(struct snd_soc_dai *dai, int mute)
 	return 0;
 }
 
+/* FLL divisors */
 struct _fll_div {
-	int div;		 
+	int div;		/* FLL_OUTDIV */
 	int n;
 	int k;
-	int ratio;		 
+	int ratio;		/* FLL_FRATIO */
 };
 
+/* The size in bits of the fll divide multiplied by 10
+ * to allow rounding later */
 #define FIXED_FLL_SIZE ((1 << 16) * 10)
 
 static inline int fll_factors(struct _fll_div *fll_div, unsigned int input,
@@ -934,9 +1011,11 @@ static inline int fll_factors(struct _fll_div *fll_div, unsigned int input,
 		do_div(Kpart, t2);
 		K = Kpart & 0xFFFFFFFF;
 
+		/* Check if we need to round */
 		if ((K % 10) >= 5)
 			K += 5;
 
+		/* Move down to proper range now rounding is done */
 		K /= 10;
 		fll_div->k = K;
 	} else
@@ -959,6 +1038,7 @@ static int wm8350_set_fll(struct snd_soc_dai *codec_dai,
 	if (freq_in == priv->fll_freq_in && freq_out == priv->fll_freq_out)
 		return 0;
 
+	/* power down FLL - we need to do this for reconfiguration */
 	wm8350_clear_bits(wm8350, WM8350_POWER_MGMT_4,
 			  WM8350_FLL_ENA | WM8350_FLL_OSC_ENA);
 
@@ -973,6 +1053,7 @@ static int wm8350_set_fll(struct snd_soc_dai *codec_dai,
 		freq_in, freq_out, fll_div.n, fll_div.k, fll_div.div,
 		fll_div.ratio);
 
+	/* set up N.K & dividers */
 	fll_1 = snd_soc_read(codec, WM8350_FLL_CONTROL_1) &
 	    ~(WM8350_FLL_OUTDIV_MASK | WM8350_FLL_RSP_RATE_MASK | 0xc000);
 	snd_soc_write(codec, WM8350_FLL_CONTROL_1,
@@ -987,6 +1068,7 @@ static int wm8350_set_fll(struct snd_soc_dai *codec_dai,
 			   fll_4 | (fll_div.k ? WM8350_FLL_FRAC : 0) |
 			   (fll_div.ratio == 8 ? WM8350_FLL_SLOW_LOCK_REF : 0));
 
+	/* power FLL on */
 	wm8350_set_bits(wm8350, WM8350_POWER_MGMT_4, WM8350_FLL_OSC_ENA);
 	wm8350_set_bits(wm8350, WM8350_POWER_MGMT_4, WM8350_FLL_ENA);
 
@@ -1029,35 +1111,43 @@ static int wm8350_set_bias_level(struct snd_soc_codec *codec,
 			if (ret != 0)
 				return ret;
 
+			/* Enable the system clock */
 			wm8350_set_bits(wm8350, WM8350_POWER_MGMT_4,
 					WM8350_SYSCLK_ENA);
 
+			/* mute DAC & outputs */
 			wm8350_set_bits(wm8350, WM8350_DAC_MUTE,
 					WM8350_DAC_MUTE_ENA);
 
+			/* discharge cap memory */
 			wm8350_reg_write(wm8350, WM8350_ANTI_POP_CONTROL,
 					 platform->dis_out1 |
 					 (platform->dis_out2 << 2) |
 					 (platform->dis_out3 << 4) |
 					 (platform->dis_out4 << 6));
 
+			/* wait for discharge */
 			schedule_timeout_interruptible(msecs_to_jiffies
 						       (platform->
 							cap_discharge_msecs));
 
+			/* enable antipop */
 			wm8350_reg_write(wm8350, WM8350_ANTI_POP_CONTROL,
 					 (platform->vmid_s_curve << 8));
 
+			/* ramp up vmid */
 			wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1,
 					 (platform->
 					  codec_current_charge << 14) |
 					 WM8350_VMID_5K | WM8350_VMIDEN |
 					 WM8350_VBUFEN);
 
+			/* wait for vmid */
 			schedule_timeout_interruptible(msecs_to_jiffies
 						       (platform->
 							vmid_charge_msecs));
 
+			/* turn on vmid 300k  */
 			pm1 = wm8350_reg_read(wm8350, WM8350_POWER_MGMT_1) &
 			    ~(WM8350_VMID_MASK | WM8350_CODEC_ISEL_MASK);
 			pm1 |= WM8350_VMID_300K |
@@ -1065,13 +1155,16 @@ static int wm8350_set_bias_level(struct snd_soc_codec *codec,
 			wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1,
 					 pm1);
 
+
+			/* enable analogue bias */
 			pm1 |= WM8350_BIASEN;
 			wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1, pm1);
 
+			/* disable antipop */
 			wm8350_reg_write(wm8350, WM8350_ANTI_POP_CONTROL, 0);
 
 		} else {
-			 
+			/* turn on vmid 300k and reduce current */
 			pm1 = wm8350_reg_read(wm8350, WM8350_POWER_MGMT_1) &
 			    ~(WM8350_VMID_MASK | WM8350_CODEC_ISEL_MASK);
 			wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1,
@@ -1084,19 +1177,23 @@ static int wm8350_set_bias_level(struct snd_soc_codec *codec,
 
 	case SND_SOC_BIAS_OFF:
 
+		/* mute DAC & enable outputs */
 		wm8350_set_bits(wm8350, WM8350_DAC_MUTE, WM8350_DAC_MUTE_ENA);
 
 		wm8350_set_bits(wm8350, WM8350_POWER_MGMT_3,
 				WM8350_OUT1L_ENA | WM8350_OUT1R_ENA |
 				WM8350_OUT2L_ENA | WM8350_OUT2R_ENA);
 
+		/* enable anti pop S curve */
 		wm8350_reg_write(wm8350, WM8350_ANTI_POP_CONTROL,
 				 (platform->vmid_s_curve << 8));
 
+		/* turn off vmid  */
 		pm1 = wm8350_reg_read(wm8350, WM8350_POWER_MGMT_1) &
 		    ~WM8350_VMIDEN;
 		wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1, pm1);
 
+		/* wait */
 		schedule_timeout_interruptible(msecs_to_jiffies
 					       (platform->
 						vmid_discharge_msecs));
@@ -1108,17 +1205,20 @@ static int wm8350_set_bias_level(struct snd_soc_codec *codec,
 				 (platform->dis_out3 << 4) |
 				 (platform->dis_out4 << 6));
 
+		/* turn off VBuf and drain */
 		pm1 = wm8350_reg_read(wm8350, WM8350_POWER_MGMT_1) &
 		    ~(WM8350_VBUFEN | WM8350_VMID_MASK);
 		wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1,
 				 pm1 | WM8350_OUTPUT_DRAIN_EN);
 
+		/* wait */
 		schedule_timeout_interruptible(msecs_to_jiffies
 					       (platform->drain_msecs));
 
 		pm1 &= ~WM8350_BIASEN;
 		wm8350_reg_write(wm8350, WM8350_POWER_MGMT_1, pm1);
 
+		/* disable anti-pop */
 		wm8350_reg_write(wm8350, WM8350_ANTI_POP_CONTROL, 0);
 
 		wm8350_clear_bits(wm8350, WM8350_LOUT1_VOLUME,
@@ -1130,6 +1230,7 @@ static int wm8350_set_bias_level(struct snd_soc_codec *codec,
 		wm8350_clear_bits(wm8350, WM8350_ROUT2_VOLUME,
 				  WM8350_OUT2R_ENA);
 
+		/* disable clock gen */
 		wm8350_clear_bits(wm8350, WM8350_POWER_MGMT_4,
 				  WM8350_SYSCLK_ENA);
 
@@ -1200,7 +1301,8 @@ static irqreturn_t wm8350_hpl_jack_handler(int irq, void *data)
 	if (device_may_wakeup(wm8350->dev))
 		pm_wakeup_event(wm8350->dev, 250);
 
-	schedule_delayed_work(&priv->hpl.work, msecs_to_jiffies(200));
+	queue_delayed_work(system_power_efficient_wq,
+			   &priv->hpl.work, msecs_to_jiffies(200));
 
 	return IRQ_HANDLED;
 }
@@ -1217,11 +1319,23 @@ static irqreturn_t wm8350_hpr_jack_handler(int irq, void *data)
 	if (device_may_wakeup(wm8350->dev))
 		pm_wakeup_event(wm8350->dev, 250);
 
-	schedule_delayed_work(&priv->hpr.work, msecs_to_jiffies(200));
+	queue_delayed_work(system_power_efficient_wq,
+			   &priv->hpr.work, msecs_to_jiffies(200));
 
 	return IRQ_HANDLED;
 }
 
+/**
+ * wm8350_hp_jack_detect - Enable headphone jack detection.
+ *
+ * @codec:  WM8350 codec
+ * @which:  left or right jack detect signal
+ * @jack:   jack to report detection events on
+ * @report: value to report
+ *
+ * Enables the headphone jack detection of the WM8350.  If no report
+ * is specified then detection is disabled.
+ */
 int wm8350_hp_jack_detect(struct snd_soc_codec *codec, enum wm8350_jack which,
 			  struct snd_soc_jack *jack, int report)
 {
@@ -1256,6 +1370,7 @@ int wm8350_hp_jack_detect(struct snd_soc_codec *codec, enum wm8350_jack which,
 		wm8350_clear_bits(wm8350, WM8350_JACK_DETECT, ena);
 	}
 
+	/* Sync status */
 	switch (which) {
 	case WM8350_JDL:
 		wm8350_hpl_jack_handler(0, priv);
@@ -1292,6 +1407,17 @@ static irqreturn_t wm8350_mic_handler(int irq, void *data)
 	return IRQ_HANDLED;
 }
 
+/**
+ * wm8350_mic_jack_detect - Enable microphone jack detection.
+ *
+ * @codec:         WM8350 codec
+ * @jack:          jack to report detection events on
+ * @detect_report: value to report when presence detected
+ * @short_report:  value to report when microphone short detected
+ *
+ * Enables the microphone jack detection of the WM8350.  If both reports
+ * are specified as zero then detection is disabled.
+ */
 int wm8350_mic_jack_detect(struct snd_soc_codec *codec,
 			   struct snd_soc_jack *jack,
 			   int detect_report, int short_report)
@@ -1383,18 +1509,22 @@ static  int wm8350_codec_probe(struct snd_soc_codec *codec)
 
 	snd_soc_codec_set_cache_io(codec, 8, 16, SND_SOC_REGMAP);
 
+	/* Put the codec into reset if it wasn't already */
 	wm8350_clear_bits(wm8350, WM8350_POWER_MGMT_5, WM8350_CODEC_ENA);
 
 	INIT_DELAYED_WORK(&codec->dapm.delayed_work, wm8350_pga_work);
 	INIT_DELAYED_WORK(&priv->hpl.work, wm8350_hpl_work);
 	INIT_DELAYED_WORK(&priv->hpr.work, wm8350_hpr_work);
 
+	/* Enable the codec */
 	wm8350_set_bits(wm8350, WM8350_POWER_MGMT_5, WM8350_CODEC_ENA);
 
+	/* Enable robust clocking mode in ADC */
 	snd_soc_write(codec, WM8350_SECURITY, 0xa7);
 	snd_soc_write(codec, 0xde, 0x13);
 	snd_soc_write(codec, WM8350_SECURITY, 0);
 
+	/* read OUT1 & OUT2 volumes */
 	out1 = &priv->out1;
 	out2 = &priv->out2;
 	out1->left_vol = (wm8350_reg_read(wm8350, WM8350_LOUT1_VOLUME) &
@@ -1410,6 +1540,7 @@ static  int wm8350_codec_probe(struct snd_soc_codec *codec)
 	wm8350_reg_write(wm8350, WM8350_LOUT2_VOLUME, 0);
 	wm8350_reg_write(wm8350, WM8350_ROUT2_VOLUME, 0);
 
+	/* Latch VU bits & mute */
 	wm8350_set_bits(wm8350, WM8350_LOUT1_VOLUME,
 			WM8350_OUT1_VU | WM8350_OUT1L_MUTE);
 	wm8350_set_bits(wm8350, WM8350_LOUT2_VOLUME,
@@ -1419,11 +1550,14 @@ static  int wm8350_codec_probe(struct snd_soc_codec *codec)
 	wm8350_set_bits(wm8350, WM8350_ROUT2_VOLUME,
 			WM8350_OUT2_VU | WM8350_OUT2R_MUTE);
 
+	/* Make sure AIF tristating is disabled by default */
 	wm8350_clear_bits(wm8350, WM8350_AI_FORMATING, WM8350_AIF_TRI);
 
+	/* Make sure we've got a sane companding setup too */
 	wm8350_clear_bits(wm8350, WM8350_ADC_DAC_COMP,
 			  WM8350_DAC_COMP | WM8350_LOOPBACK);
 
+	/* Make sure jack detect is disabled to start off with */
 	wm8350_clear_bits(wm8350, WM8350_JACK_DETECT,
 			  WM8350_JDL_ENA | WM8350_JDR_ENA);
 
@@ -1438,6 +1572,7 @@ static  int wm8350_codec_probe(struct snd_soc_codec *codec)
 	wm8350_register_irq(wm8350, WM8350_IRQ_CODEC_MICD,
 			    wm8350_mic_handler, 0, "Microphone detect", priv);
 
+
 	wm8350_set_bias_level(codec, SND_SOC_BIAS_STANDBY);
 
 	return 0;
@@ -1464,6 +1599,8 @@ static int  wm8350_codec_remove(struct snd_soc_codec *codec)
 	cancel_delayed_work_sync(&priv->hpl.work);
 	cancel_delayed_work_sync(&priv->hpr.work);
 
+	/* if there was any work waiting then we run it now and
+	 * wait for its completion */
 	flush_delayed_work(&codec->dapm.delayed_work);
 
 	wm8350_set_bias_level(codec, SND_SOC_BIAS_OFF);
